﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Drawing;
using System.Linq;
using System.Windows.Threading;
using agsXMPP.protocol.iq.vcard;
using VSTalk.Engine.Core.Context;
using VSTalk.Engine.Core.Feed;
using VSTalk.Engine.Core.XmppConnection;
using VSTalk.Engine.Core.XmppUtils;
using VSTalk.Model;
using VSTalk.Tools;

using agsXMPP;
using agsXMPP.protocol.client;

using ShowType = VSTalk.Model.ShowType;

namespace VSTalk.Engine.Core.EntitySession
{
    public class InterlocutorContext : IInterlocutorContext
    {
        private readonly XmppClientConnection _connection;

        private readonly Client _client;

        private IActionWrapper<object, Presence> _onPresence;

        private IActionWrapper<object, IQ> _onIq;

        private IListenerUnit _resourcesUnit;

        private IListenerUnit _clientContextUnit;

        private IListenerUnit _avatarUnit;

        private IListenerUnit _avatarChangedUnit;

        private string _lastHashValue;

        public IInterlocutorAvatar Avatar { get; private set; }

        private IClientContext ClientContext
        {
            get { return AppServices.Get<ISessionModel>().GetClientContext(_client); }
        }

        private static IConnectionManager ConnectionManager
        {
            get { return AppServices.Get<IConnectionManager>(); }
        }

        private static IModelContext ModelContext
        {
            get { return AppServices.Get<IModelContext>(); }
        }

        public Jid Bare { get; private set; }

        public ContactState State
        {
            get
            {
                if (!Resources.Any())
                {
                    return ContactState.Offline;
                }
                return Resources.Any(resource => resource.State == ContactState.Online)
                        ? ContactState.Online
                        : ContactState.Offline;
            }
        }

        public ShowType Show
        {
            get
            {
                if (!Resources.Any())
                {
                    return ShowType.NONE;
                }
                return Resources.First().Show;
            }
        }

        public Interlocutor Interlocutor { get; private set; }

        public ObservableCollection<IInterlocutorResource> Resources { get; private set; }

        public InterlocutorContext(Interlocutor interlocutor)
        {
            Interlocutor = interlocutor;
            Bare = XmppIdConverter.Jid(interlocutor);
            Resources = new ObservableCollection<IInterlocutorResource>();

            _client = ModelContext.GetClientByContact(interlocutor);
            _connection = ConnectionManager
                    .GetOrCreateConnection(_client)
                    .Connection;

            _connection.Send(new VcardIq(IqType.get, Bare, ClientContext.Jid));
        }

        private void OnIq(object sender, IQ iq)
        {
            if (iq.Type == IqType.result && AddressPatterns.To(_client, Interlocutor).Invoke(iq))
                if (iq.Vcard != null && iq.Vcard.Photo != null)
                {
                    Avatar = new InterlocutorAvatar(Bare, iq.Vcard.Photo.ImageFormat);
                    if (_avatarChangedUnit == null) _avatarChangedUnit = Avatar.SubscribeToChange(() => Avatar.Avatar, obj => Avatar.Avatar != null, avatar => new Bitmap(avatar.Avatar).Save(avatar.AvatarFileName, avatar.ImageExtension));
                    Avatar.Avatar = iq.Vcard.Photo.Image;
                    if (_lastHashValue != null)
                    {
                        Avatar.Hash = _lastHashValue;
                        _lastHashValue = null;
                    }
                    if (_avatarUnit == null)
                        _avatarUnit = Avatar.SubscribeToChange(() => Avatar.Hash, obj => Avatar.Hash != null,
                                avatar => _connection.Send(new VcardIq(IqType.get, Bare, ClientContext.Jid)));
                }
        }

        public void StartListen()
        {
            Subscribe();
            StartHandling();
        }

        public void StopListen()
        {
            StopHandling();
            Unsubscribe();
        }

        private void Subscribe()
        {
            var dispatcher = Dispatcher.CurrentDispatcher;
            _onPresence = dispatcher.Wrap<object, Presence>(OnPresence);
            _connection.OnPresence += _onPresence.Exec;
            _onIq = dispatcher.Wrap<object, IQ>(OnIq);
            _connection.OnIq += _onIq.Exec;
        }

        private void StartHandling()
        {
            _resourcesUnit = Resources.HandleCollectionChanges(initialOrAdd:(resource, p) =>
            {
                PropertyChanged.Notify(() => State);
                PropertyChanged.Notify(() => Show);

                p.Add(resource.SubscribeToChange(() => resource.State, _ => PropertyChanged.Notify(() => State)));
                p.Add(resource.SubscribeToChange(() => resource.Show, _ => PropertyChanged.Notify(() => Show)));
            }, removed: () =>
            {
                PropertyChanged.Notify(() => State);
                PropertyChanged.Notify(() => Show);
            });

            _clientContextUnit = ClientContext.SubscribeToChange(() => ClientContext.State,
                sender => ClientContext.State == ClientState.Disconnected,
                sender =>
                {
                    foreach (var source in Resources.ToList())
                    {
                        Resources.Remove(source);
                    }
                });
            
        }

        private void StopHandling()
        {
            _clientContextUnit.Stop();
            _resourcesUnit.Stop();
            _avatarUnit.Stop();
            _avatarChangedUnit.Stop();
        }

        private void Unsubscribe()
        {
            _connection.OnPresence -= _onPresence.Exec;
            _connection.OnIq -= _onIq.Exec;
        }

        private void OnPresence(object sender, Presence p)
        {
            if (!string.Equals(p.From.Bare, Bare, StringComparison.InvariantCultureIgnoreCase)) return;
            var resource = GetOrCreateResource(p.From);
            resource.State = p.Type == PresenceType.available ? ContactState.Online : ContactState.Offline;
            resource.Show = (ShowType)Enum.Parse(typeof(ShowType), p.Show.ToString());
            resource.Status = p.Status;
            CheckHashUpdates(p);
        }

        private void CheckHashUpdates(Presence p)
        {
            if (p.From.Bare.Equals(Bare) && p.HasTag("photo",true))
                if(Avatar != null)
                    Avatar.Hash = p.GetTag("photo",true);
                else
                    _lastHashValue = p.GetTag("photo", true);
            
        }

        private IInterlocutorResource GetOrCreateResource(Jid jid)
        {
            return Resources.FirstOrDefault(resource => Equals(resource.Jid, jid)) ??
                   CreateResource(jid);
        }

        private IInterlocutorResource CreateResource(Jid jid)
        {
            var resource = new InterlocutorResource(jid);
            Resources.Add(resource);
            return resource;
        }

        public event PropertyChangedEventHandler PropertyChanged;
    }
}