﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Reactive.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;

namespace VSTalk.Engine.Utils
{
    public class TreeViewHelper
    {
        public static bool GetExpanded(DependencyObject obj)
        {
            return (bool)obj.GetValue(ExpandedProperty);
        }

        public static void SetExpanded(DependencyObject obj, bool value)
        {
            obj.SetValue(ExpandedProperty, value);
        }

        // Using a DependencyProperty as the backing store for Expanded.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ExpandedProperty =
            DependencyProperty.RegisterAttached("Expanded", typeof(bool), typeof(TreeViewHelper), new PropertyMetadata(false, ExpandNewNodes));

        private static void ExpandNewNodes(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs dependencyPropertyChangedEventArgs)
        {
            var treeView = (TreeView)dependencyObject;
            var generator = treeView.ItemContainerGenerator;
            generator.ItemsChanged += (sender, args) =>
            {
                if (args.Action == NotifyCollectionChangedAction.Reset)
                {
                    Observable.FromEventPattern<EventHandler, EventArgs>(h => generator.StatusChanged += h,
                            h => generator.StatusChanged -= h)
                            .SkipWhile(eventArgs => generator.Status != GeneratorStatus.ContainersGenerated)
                            .Take(1)
                            .Subscribe(eventArgs => ExpandAll(treeView));
                }
                else if (args.Action == NotifyCollectionChangedAction.Add)
                {
                    Observable.FromEventPattern<EventHandler, EventArgs>(h => generator.StatusChanged += h,
                            h => generator.StatusChanged -= h)
                            .SkipWhile(eventArgs => generator.Status != GeneratorStatus.ContainersGenerated)
                            .Take(1)
                            .Subscribe(eventArgs => ExpandSpecific(treeView, args));
                }
            };
        }

        private static void ExpandSpecific(TreeView treeView, ItemsChangedEventArgs args)
        {
            var container = treeView.ItemContainerGenerator.ContainerFromIndex(args.Position.Offset + args.Position.Index);
            var item = container as TreeViewItem;
            if (item == null)
            {
                return;
            }
            item.IsExpanded = true;
        }

        private static void ExpandAll(TreeView treeView)
        {
            foreach (object obj in treeView.Items)
            {
                var childControl = treeView.ItemContainerGenerator.ContainerFromItem(obj) as ItemsControl;
                var item = childControl as TreeViewItem;
                if (item != null)
                {
                    item.IsExpanded = true;
                }
            }
        }

        private static Dictionary<DependencyObject, TreeViewSelectedItemBehavior> behaviors = new Dictionary<DependencyObject, TreeViewSelectedItemBehavior>();

        public static object GetSelectedItem(DependencyObject obj)
        {
            return (object)obj.GetValue(SelectedItemProperty);
        }

        public static void SetSelectedItem(DependencyObject obj, object value)
        {
            obj.SetValue(SelectedItemProperty, value);
        }

        // Using a DependencyProperty as the backing store for SelectedItem.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty SelectedItemProperty =
                DependencyProperty.RegisterAttached("SelectedItem",
                typeof(object),
                typeof(TreeViewHelper),
                new FrameworkPropertyMetadata(new object(), SelectedItemChanged));

        private static void SelectedItemChanged(DependencyObject obj, DependencyPropertyChangedEventArgs e)
        {
            if (!(obj is TreeView))
                return;

            if (!behaviors.ContainsKey(obj))
                behaviors.Add(obj, new TreeViewSelectedItemBehavior(obj as TreeView));
        }

        private class TreeViewSelectedItemBehavior
        {
            public TreeViewSelectedItemBehavior(TreeView view)
            {
                view.SelectedItemChanged += (sender, e) => SetSelectedItem(view, e.NewValue);
            }
        }
    }
}