using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Configuration;
using System.Linq;

using VSTalk.Engine.ViewModel.ContactList.ClientResources;
using VSTalk.Engine.ViewModel.ContactList.Filter;
using VSTalk.Model;
using VSTalk.Tools;

namespace VSTalk.Engine.ViewModel.ContactList
{
    public class ClientResourcesManager : IResourceUpdateHandler, IListener
    {
        private readonly IResourcePresenter _presenter;

        private readonly NullClientResource _nullResource;

        private readonly IList<IClientResource> _registeredResources;

        private readonly IList<GenericResourceFilter> _filters = new List<GenericResourceFilter>();
        private SettingsSavingEventHandler _settingSaving;
        private IListenerUnit _roomUnit;
        private IListenerUnit _contactUnit;

        private ObservableCollection<IClientResource> ActiveResources
        {
            get { return _presenter.ActiveResources; }
        }

        private Client Client
        {
            get { return _presenter.Model; }
        }

        public ClientResourcesManager(IResourcePresenter presenter)
        {
            _presenter = presenter;
            _nullResource = new NullClientResource();
            ActiveResources.Add(_nullResource);
            _registeredResources = new ObservableCollection<IClientResource>();
            PopulateResources();
            Update();
        }


        public void StartListen()
        {
            _settingSaving = delegate { Update(); };
            Properties.Settings.Default.SettingsSaving += _settingSaving;

            _roomUnit = Client.Rooms.HandleAddRemoveChanges(
                                     added: room => RegisterResource(new RoomResourceViewModel(room)),
                                     removed: room => UnRegisterResource(_registeredResources.FirstOrDefault(resource => resource.Model == room)));
            _contactUnit = Client.Contacts.HandleAddRemoveChanges(
                                        added: interlocutor => RegisterResource(new InterlocutorResourceViewModel(interlocutor)),
                                        removed: interlocutor => UnRegisterResource(_registeredResources.FirstOrDefault(resource => resource.Model == interlocutor)));
        }

        public void StopListen()
        {
            Properties.Settings.Default.SettingsSaving -= _settingSaving;
            _roomUnit.Stop();
            _contactUnit.Stop();
        }

        private void RegisterResource(IClientResource resource)
        {
            _registeredResources.Add(resource);
            resource.StartListen();
            resource.Subscribe(this);
            if (IsAccepted(resource))
            {
                AddResource(resource);
            }
        }

        private void UnRegisterResource(IClientResource resource)
        {
            if (resource == null)
            {
                return;
            }
            resource.StopListen();
            _registeredResources.Remove(resource);
            if (ActiveResources.Contains(resource))
            {
                RemoveResource(resource);
            }
        }

        private void PopulateResources()
        {
            foreach (var room in Client.Rooms)
            {
                RegisterResource(new RoomResourceViewModel(room));
            }

            foreach (var contact in Client.Contacts)
            {
                RegisterResource(new InterlocutorResourceViewModel(contact));
            }
        }

        private void AddResource(IClientResource targetResource)
        {
            if (ActiveResources.Contains(_nullResource))
            {
                ActiveResources.Remove(_nullResource);
            }
            var position = Position(ActiveResources, targetResource);
            ActiveResources.Insert(position, targetResource);
        }

        private void RemoveResource(IClientResource resource)
        {
            ActiveResources.Remove(resource);
            if (ActiveResources.Count == 0)
            {
                ActiveResources.Add(_nullResource);
            }
        }

        private static int Position(IEnumerable<IClientResource> resources, IClientResource target)
        {
            if (resources.Contains(target))
            {
                return OrderedResources(resources)
                        .TakeWhile(resource => resource != target).Count();
            }
            var concat = resources.Concat(new[]
            {
                    target
            });
            return OrderedResources(concat).TakeWhile(resource => resource != target).Count();
        }

        private static IEnumerable<IClientResource> OrderedResources(IEnumerable<IClientResource> resources)
        {
            return resources.OrderBy(resource => resource.GetType().Name)
                            .ThenBy(resource => resource.StateOrder)
                            .ThenBy(resource => resource.Name);
        }

        private bool IsAccepted(IClientResource clientResource)
        {
            var param = new FilterParam();
            foreach (var filter in _filters)
            {
                filter.Apply(clientResource, param);
                if (param.Result == FilterResult.Reject)
                {
                    return false;
                }
            }
            return true;
        }

        public void Update()
        {
            //remove
            var toRemove = ActiveResources.Where(resource => !IsAccepted(resource)).ToList();
            foreach (var resource in toRemove)
            {
                RemoveResource(resource);
            }

            //insert
            var toInsert = _registeredResources.Exclude(toRemove.Union(ActiveResources));
            foreach (var clientResource in toInsert.Where(IsAccepted))
            {
                AddResource(clientResource);
            }

            //reorder
            var orderedResorces = OrderedResources(ActiveResources).ToList();
            foreach (var target in orderedResorces)
            {
                var actualPosition = ActiveResources.IndexOf(target);
                var expectedPosition = orderedResorces.IndexOf(target);
                if (actualPosition != expectedPosition)
                {
                    var item = ActiveResources[actualPosition];
                    ActiveResources.Remove(item);
                    ActiveResources.Insert(expectedPosition, item);
                }
            }
        }

        public void AddFilter(GenericResourceFilter filter)
        {
            _filters.Add(filter);
            Update();
        }
    }
}