using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;

using VSTalk.Engine.Core.Context;
using VSTalk.Engine.Core.EntitySession;
using VSTalk.Engine.Core.EnvironmentManagers.UI;
using VSTalk.Engine.Core.XmppCommunication;
using VSTalk.Engine.Core.XmppCommunication.Services.Authorization;
using VSTalk.Engine.Core.XmppCommunication.Services.ContactManager;
using VSTalk.Engine.Core.XmppUtils;
using VSTalk.Engine.Utils;
using VSTalk.Engine.ViewModel.ContactList.ClientResources;
using VSTalk.Engine.ViewModel.History;
using VSTalk.Engine.Windows;
using VSTalk.Model;
using VSTalk.Tools;

namespace VSTalk.Engine.ViewModel.ContactList
{
    public class ContactListViewModel : INotifyPropertyChanged, IListener
    {
        public static IModelContext ModelContext
        {
            get { return AppServices.Get<IModelContext>(); }
        }

        public static ISessionModel SessionModel
        {
            get { return AppServices.Get<ISessionModel>(); }
        }

        private static IWindowManager WindowManager
        {
            get { return AppServices.Get<IWindowManager>(); }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        private ObservableCollection<ClientNodeViewModel> _clients = new ObservableCollection<ClientNodeViewModel>();
        
        private IListenerUnit _clientUnit;

        public ObservableCollection<ClientNodeViewModel> Clients
        {
            get { return _clients; }
            set
            {
                _clients = value;
                PropertyChanged.Notify(() => Clients);
            }
        }

        public RelayCommand<Interlocutor> ShowInterlocutorChat { get; private set; }

        public RelayCommand<Interlocutor> ShowContactLog { get; private set; }

        public RelayCommand<Interlocutor> SendRequestAuthorization { get; private set; }
        
        public RelayCommand<Interlocutor> RemoveContact { get; private set; }

        public RelayCommand<Interlocutor> ShowContactProfile { get; private set; }

        public ContactListViewModel()
        {
            SetCommands();
        }

        public void StartListen()
        {
            _clientUnit = ModelContext.Clients.HandleCollectionChanges(initialOrAdd: AddClientPresentation,
                    removed: RemoveClientPresentation);
        }

        public void StopListen()
        {
            _clientUnit.Stop();
        }

        private void AddClientPresentation(Client client)
        {
            var clientNodeViewModel = new ClientNodeViewModel(client);
            Clients.Add(clientNodeViewModel);
            clientNodeViewModel.StartListen();
        }

        private void RemoveClientPresentation(Client client)
        {
            var clientNodeViewModel = Clients.First(model => model.Model == client);
            clientNodeViewModel.StopListen();
            Clients.Remove(clientNodeViewModel);
        }

        private void SetCommands()
        {
            ShowInterlocutorChat = new RelayCommand<Interlocutor>(ShowInterlocutorChatExecuted);
            ShowContactLog = new RelayCommand<Interlocutor>(ShowContactLogExecuted);
            SendRequestAuthorization = new RelayCommand<Interlocutor>(SendRequestAuthorizationExecuted, SendRequestAuthorizationCanExecute);
            RemoveContact = new RelayCommand<Interlocutor>(RemoveContactExecuted, RemoveContactCanExecute);
            ShowContactProfile = new RelayCommand<Interlocutor>(ShowContactProfileExecuted);
        }

        protected void RemoveContactExecuted(Interlocutor interlocutor)
        {
            var contactManager = CommunicationServiceFactory.Create<ContactManager>(interlocutor);
            contactManager.RemoveContact(interlocutor);
        }

        protected bool RemoveContactCanExecute(Interlocutor interlocutor)
        {
            if (interlocutor == null) return false;
            var client = ModelContext.GetClientByContact(interlocutor);
            if (client == null) return false;
            var context = SessionModel.GetClientContext(client);
            return context.State == ClientState.Connected;
        }

        protected void SendRequestAuthorizationExecuted(Interlocutor interlocutor)
        {
            var authService = CommunicationServiceFactory.Create<AuthorizationProvider>(interlocutor);
            authService.SendAuthorizationRequest(XmppIdConverter.Jid(interlocutor));
        }

        protected bool SendRequestAuthorizationCanExecute(Interlocutor interlocutor)
        {
            if (interlocutor == null) return false;
            var client = ModelContext.GetClientByContact(interlocutor);
            if (client == null) return false;
            var context = SessionModel.GetClientContext(client);
            return !interlocutor.Subscribed && context.State == ClientState.Connected;
        }

        protected void ShowContactLogExecuted(Interlocutor interlocutor)
        {
            var client = ModelContext.GetClientByContact(interlocutor);
            var bare = XmppIdConverter.Jid(interlocutor).Bare;
            var vm = new HistoryExplorerViewModel(client, bare);
            var wnd = new HistoryExplorerWindow();
            wnd.DataContext = vm;
            vm.StartListen();
            wnd.Closed += (sender, args) => vm.StopListen();
            wnd.Show();
        }

        protected void ShowInterlocutorChatExecuted(Interlocutor interlocutor)
        {
            WindowManager.ShowChatWindow(interlocutor);
        }

        protected void ShowContactProfileExecuted(Interlocutor interlocutor)
        {
            var interlocutorSummary = new InterlocuterSummaryViewModel(interlocutor);
            var summaryWindow = new InterlocutorSummaryWindow();
            summaryWindow.DataContext = interlocutorSummary;
            summaryWindow.Show();
        }
    }
}