using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Media;

using VSTalk.Engine.Annotations;
using VSTalk.Engine.Core.ChatSession;
using VSTalk.Engine.Core.Context;
using VSTalk.Engine.Core.EntitySession;
using VSTalk.Engine.Core.EnvironmentManagers.UI;
using VSTalk.Engine.Core.Feed;
using VSTalk.Engine.Core.Feed.MessageState;
using VSTalk.Engine.Core.XmppCommunication;
using VSTalk.Engine.Core.XmppCommunication.Services.MessageService;
using VSTalk.Engine.Utils;
using VSTalk.Engine.ViewModel.Messenger.MessageBuilder;
using VSTalk.Engine.ViewModel.Messenger.ThreadRenderer;
using VSTalk.Tools;
using VSTalk.Model;

namespace VSTalk.Engine.ViewModel.Messenger
{
    public class InterlocutorChatViewModel : IChatViewModel, INotifyPropertyChanged
    {
        private readonly IFrameViewModel _frame;

        private readonly Interlocutor _interlocutor;

        private RichTextBox _chatTextBox;

        private RichTextBox _messageTextBox;

        private readonly IList<IMessage> _pendingMessages = new List<IMessage>();

        private readonly EntityThread _entityThread;

        private IListenerUnit _frameUnit;
        private IListenerUnit _themeUnit;

        private static IModelContext ModelContext
        {
            get { return AppServices.Get<IModelContext>(); }
        }

        private static IMessageFeed MessageFeed
        {
            get { return AppServices.Get<IMessageFeed>(); }
        }

        private static ISessionModel SessionModel
        {
            get { return AppServices.Get<ISessionModel>(); }
        }

        private static IChatSessionFactory ChatSession
        {
            get { return AppServices.Get<IChatSessionFactory>(); }
        }

        private static IWindowManager WindowManager
        {
            get { return AppServices.Get<IWindowManager>(); }
        }

        private static IThemeManager ThemeManager
        {
            get { return WindowManager.ThemeManager; }
        }

        #region properties
        public Interlocutor Interlocutor
        {
            get { return _interlocutor; }
        }

        public RichTextBox ChatTextBox
        {
            get { return _chatTextBox; }
            set
            {
                _chatTextBox = value;
                PropertyChanged.Notify(() => ChatTextBox);
            }
        }

        public RichTextBox MessageTextBox
        {
            get { return _messageTextBox; }
            set
            {
                _messageTextBox = value;
                PropertyChanged.Notify(() => MessageTextBox);
            }
        }

        [UsedImplicitly]
        public RelayCommand PasteCallStack
        {
            get { return _frame.PasteCallStack; }
        }

        [UsedImplicitly]
        public RelayCommand PasteDebugOutput
        {
            get { return _frame.PasteDebugOutput; }
        }

        #endregion

        public InterlocutorChatViewModel(IFrameViewModel frame, Interlocutor interlocutor)
        {
            _frame = frame;
            _interlocutor = interlocutor;

            var chatDocument = new FlowDocument();
            ChatTextBox = new RichTextBox(chatDocument);
            ChatTextBox.VerticalScrollBarVisibility = ScrollBarVisibility.Auto;
            ChatTextBox.Background = Brushes.Transparent;
            ChatTextBox.IsReadOnly = true;
            _entityThread = CreateThread(chatDocument);

            MessageTextBox = new RichTextBox(new FlowDocument());
            MessageTextBox.Background = Brushes.Transparent;
            MessageTextBox.VerticalScrollBarVisibility = ScrollBarVisibility.Auto;
            ParseCurrentHistory();
            SetTheme(ThemeManager.ActiveTheme);
        }

        public void StartListen()
        {
            _frameUnit = _frame.SubscribeToChange(() => _frame.CurrentChat, sender =>
            {
                if (_frame.CurrentChat == this)
                {
                    foreach (var pendingMessage in _pendingMessages)
                    {
                        var counter = GetCounter();
                        counter.MarkAsRead(pendingMessage);
                    }
                    _pendingMessages.Clear();
                    ChatTextBox.ScrollToEnd();
                }
            });

            _themeUnit = ThemeManager.SubscribeToChange(() => ThemeManager.ActiveTheme, ThemeChanged);
        }

        private void ThemeChanged(IThemeManager sender)
        {
            SetTheme(ThemeManager.ActiveTheme);
        }

        private void SetTheme(ThemeMode activeTheme)
        {
            ChatTextBox.Document.Foreground = MessageTextBox.Document.Foreground = 
                activeTheme == ThemeMode.Dark
                ? Brushes.White
                : Brushes.Black;
        }

        public void StopListen()
        {
            _frameUnit.Stop();
            _themeUnit.Stop();
        }

        private EntityThread CreateThread(FlowDocument document)
        {
            var client = ModelContext.GetClientByContact(Interlocutor);
            var thread = new EntityThread(document);
            thread.Renderers.Add(new ContentMessageRenderer(thread, client));
            thread.Renderers.Add(new StatusMessageRenderer(thread, client));
            thread.Renderers.Add(new SubjectMessageRenderer(thread, client));
            return thread;
        }

        protected void ParseCurrentHistory()
        {
            ModifyTextBox(() =>
            {
                foreach (var msg in MessageFeed.GetMessages(Predicate))
                {
                    var counter = GetCounter();
                    counter.MarkAsRead(msg);
                    _entityThread.Push(msg);
                }
            });
        }

        protected void ModifyTextBox(Action action)
        {
            ChatTextBox.BeginChange();
            action();
            ChatTextBox.EndChange();
            ChatTextBox.ScrollToEnd();
        }

        public object Model
        {
            get { return Interlocutor; }
        }

        public void SendMessage()
        {
            var chatSession = ChatSession.Create(Interlocutor);
            var messageBuilder = MessageBuilderFactory.Create(chatSession);
            var document = MessageTextBox.Document;
            var xmppMessage = messageBuilder.Create(document);

            var messageService = CommunicationServiceFactory.Create<MessageService>(Interlocutor);
            messageService.SendMessage(chatSession, xmppMessage);
            ClearMessage();
        }

        public bool CanSendMessage()
        {
            var client = ModelContext.GetClientByContact(Interlocutor);
            var context = SessionModel.GetClientContext(client);
            return context.State == ClientState.Connected &&
                   !IsMessageEmpty();

        }

        public void AppendToMessage(string str)
        {
            MessageTextBox.Document.Blocks.Add(new Paragraph(new Run(str)));
        }

        private void ClearMessage()
        {
            MessageTextBox.Document.Blocks.Clear();
        }

        private bool IsMessageEmpty()
        {
            return new TextRange(MessageTextBox.Document.ContentStart, MessageTextBox.Document.ContentEnd).IsEmpty;
        }

        public void Consume(IMessage message)
        {
            if (!_frame.IsActive || _frame.CurrentChat != this)
            {
                _pendingMessages.Add(message);
            }
            else if (_frame.CurrentChat == this)
            {
                var counter = GetCounter();
                counter.MarkAsRead(message);
            }

            ModifyTextBox(() => _entityThread.Push(message));
        }

        public UnreadMessageCounter GetCounter()
        {
            var stateManager = AppServices.Get<IMessageSateManager>();
            var counter = stateManager.GetCounter(_interlocutor);
            return counter;
        }

        public bool Predicate(IMessage message)
        {
            return AddressPatterns.From(_interlocutor)(message) ||
                   AddressPatterns.To(_interlocutor)(message);
        }

        public event PropertyChangedEventHandler PropertyChanged;
    }
}