﻿using System;
using System.ComponentModel;
using System.Windows.Controls;
using System.Windows.Documents;

using VSTalk.Engine.Annotations;
using VSTalk.Engine.Core.Context;
using VSTalk.Engine.Core.EntitySession;
using VSTalk.Engine.Core.EnvironmentManagers.UI;
using VSTalk.Engine.Core.Feed;
using VSTalk.Engine.Core.XmppCommunication;
using VSTalk.Engine.Core.XmppCommunication.Services.MessageService;
using VSTalk.Engine.Utils;
using VSTalk.Engine.ViewModel.Messenger.MessageBuilder;
using VSTalk.Engine.ViewModel.Messenger.ThreadRenderer;
using VSTalk.Model;

using VSTalk.Tools;

namespace VSTalk.Engine.ViewModel.Messenger
{
    public class RoomChatViewModel : IChatViewModel, INotifyPropertyChanged
    {
        private readonly IFrameViewModel _frame;

        private readonly Room _room;

        private RichTextBox _chatTextBox;

        private RichTextBox _messageTextBox;

        private readonly EntityThread _entityThread;
        private IListenerUnit _themeUnit;
        private IListenerUnit _frameUnit;

        public object Model
        {
            get { return Room; }
        }

        private IModelContext ModelContext
        {
            get { return AppServices.Get<IModelContext>(); }
        }

        private IMessageFeed MessageFeed
        {
            get { return AppServices.Get<IMessageFeed>(); }
        }

        private static ISessionModel SessionModel
        {
            get { return AppServices.Get<ISessionModel>(); }
        }

        private static IWindowManager WindowManager
        {
            get { return AppServices.Get<IWindowManager>(); }
        }

        private static IThemeManager ThemeManager
        {
            get { return WindowManager.ThemeManager; }
        }

        public Room Room
        {
            get { return _room; }
        }

        [UsedImplicitly]
        public IRoomContext Context
        {
            get { return SessionModel.GetRoomContext(Room); }
        }

        public RichTextBox ChatTextBox
        {
            get { return _chatTextBox; }
            set
            {
                _chatTextBox = value;
                PropertyChanged.Notify(() => ChatTextBox);
            }
        }

        public RichTextBox MessageTextBox
        {
            get { return _messageTextBox; }
            set
            {
                _messageTextBox = value;
                PropertyChanged.Notify(() => MessageTextBox);
            }
        }

        public RelayCommand PasteCallStack
        {
            get { return _frame.PasteCallStack; }
        }

        public RelayCommand PasteDebugOutput
        {
            get { return _frame.PasteDebugOutput; }
        }

        public RoomChatViewModel(IFrameViewModel frame, Room room)
        {
            _frame = frame;
            _room = room;

            var chatDocument = new FlowDocument();
            ChatTextBox = new RichTextBox(chatDocument);
            ChatTextBox.VerticalScrollBarVisibility = ScrollBarVisibility.Auto;
            ChatTextBox.Background = System.Windows.Media.Brushes.Transparent;
            ChatTextBox.IsReadOnly = true;
            _entityThread = CreateThread(chatDocument);

            MessageTextBox = new RichTextBox(new FlowDocument());
            MessageTextBox.Background = System.Windows.Media.Brushes.Transparent;
            MessageTextBox.VerticalScrollBarVisibility = ScrollBarVisibility.Auto;

            ParseCurrentHistory();
            SetTheme(ThemeManager.ActiveTheme);
        }


        public void StartListen()
        {
            _themeUnit = ThemeManager.SubscribeToChange(() => ThemeManager.ActiveTheme, ThemeChanged);
            _frameUnit = _frame.SubscribeToChange(() => _frame.CurrentChat, sender =>
            {
                if (_frame.CurrentChat == this)
                {
                    ChatTextBox.ScrollToEnd();
                }
            });
        }

        private void ThemeChanged(IThemeManager sender)
        {
            SetTheme(ThemeManager.ActiveTheme);
        }

        private void SetTheme(ThemeMode activeTheme)
        {
            ChatTextBox.Document.Foreground = MessageTextBox.Document.Foreground =
                activeTheme == ThemeMode.Dark
                ? System.Windows.Media.Brushes.White
                : System.Windows.Media.Brushes.Black;
        }

        public void StopListen()
        {
            _themeUnit.Stop();
            _frameUnit.Stop();
        }

        private EntityThread CreateThread(FlowDocument document)
        {
            var client = ModelContext.GetClientByRoom(Room);
            var thread = new EntityThread(document);
            thread.Renderers.Add(new ContentMessageRenderer(thread, client));
            thread.Renderers.Add(new StatusMessageRenderer(thread, client));
            thread.Renderers.Add(new SubjectMessageRenderer(thread, client));
            return thread;
        }

        protected void ParseCurrentHistory()
        {
            ModifyTextBox(() =>
            {
                foreach (var msg in MessageFeed.GetMessages(Predicate))
                {
                    _entityThread.Push(msg);
                }
            });
        }

        protected void ModifyTextBox(Action action)
        {
            ChatTextBox.BeginChange();
            action();
            ChatTextBox.EndChange();
            ChatTextBox.ScrollToEnd();
        }

        public void SendMessage()
        {
            var messageBuilder = MessageBuilderFactory.Create(Room);
            var document = MessageTextBox.Document;
            var xmppMessage = messageBuilder.Create(document);

            var messageService = CommunicationServiceFactory.Create<MessageService>(Room);
            messageService.SendMessage(Room, xmppMessage);
            ClearMessage();
        }

        public bool CanSendMessage()
        {
            var client = ModelContext.GetClientByRoom(Room);
            var context = AppServices.Get<ISessionModel>().GetClientContext(client);
            return context.State == ClientState.Connected &&
                   !IsMessageEmpty();

        }

        public void AppendToMessage(string str)
        {
            MessageTextBox.Document.Blocks.Add(new Paragraph(new Run(str)));
        }

        private void ClearMessage()
        {
            MessageTextBox.Document.Blocks.Clear();
        }

        private bool IsMessageEmpty()
        {
            return new TextRange(MessageTextBox.Document.ContentStart, MessageTextBox.Document.ContentEnd).IsEmpty;
        }

        public void Consume(IMessage message)
        {
            ModifyTextBox(() =>
            {
                _entityThread.Push(message);
            });
        }

        public bool Predicate(IMessage message)
        {
            return AddressPatterns.From(_room)(message) ||
                   AddressPatterns.To(_room)(message);

        }

        public event PropertyChangedEventHandler PropertyChanged;
    }
}